# -*- coding: utf-8 -*-


import os
import subprocess
from collections import namedtuple
from typing import List

from pytcm.options import Option

CommandResult = namedtuple("CommandResult", ["out", "err", "returncode"])


def execute(binary: str, opts: List[Option] = ..., cwd: str = ...) -> CommandResult:
    """Execute a command using options.

    WARNING: letting users specify `binary` is unsafe
    """
    opts = opts if opts != ... else []
    cwd = cwd if cwd != ... else os.getcwd()
    c = [binary]
    c.extend([opt.parse() for opt in opts])

    proc = subprocess.Popen(c, stdout=subprocess.PIPE, stderr=subprocess.PIPE, cwd=cwd)
    out, err = proc.communicate()

    return CommandResult(out.decode(), err.decode(), proc.returncode)



# -*- coding: utf-8 -*-


from abc import ABC, abstractmethod
from dataclasses import dataclass


class Option(ABC):
    def __init__(self, value: any = ...) -> None:
        self.value = value

    def parse(self) -> str:
        return "" if self.value is ... else str(self)

    @abstractmethod
    def __str__(self) -> str:
        raise NotImplementedError


class Flag(Option):
    """A boolean option

    e.g.: --verbose
    """

    def __init__(self, abbreviation: str, value: any = ...) -> None:
        self.abbreviation = abbreviation
        super().__init__(value)

    def __str__(self) -> str:
        return self.abbreviation


@dataclass
class Positional(Option):
    """A simple inline option

    e.g.: example.txt
    """

    def __str__(self) -> str:
        return self.value


@dataclass
class Implicit(Option):
    """An option separated by a space character

    e.g.: --exclude example.txt
    """

    abbreviation: str
    value: str

    def __str__(self) -> str:
        return f"{self.abbreviation} {self.value}"


@dataclass
class Explicit(Option):
    """An option with an equal sign

    e.g.: --exclude=example.txt
    """

    abbreviation: str
    value: str

    def __str__(self) -> str:
        return f"{self.abbreviation}={self.value}"



# -*- coding: utf-8 -*-


import os
from copy import deepcopy
from typing import List

from pytcm.options import Option
from pytcm.utils import execute


class Command:
    def __init__(self, binary: str, opts: List[Option] = ..., cwd: str = ...) -> None:
        """An object that holds the context for a command.

        Parameters
        ----------
            binary: str
                path to the binary to execute
            opts: List[Option]
                list of options to pass to the binary
            cwd: str
                path to the directory where the command is executed
        """
        self._binary = binary
        self._opts = opts if opts != ... else []
        self._cwd = cwd if cwd != ... else os.getcwd()
        self._out = None
        self._err = None
        self._returncode = None

    def execute(self) -> None:
        """Execute the command. Sets the out, err and returncode properties."""
        self._out, self._err, self._returncode = execute(
            self._binary, self._opts, self._cwd
        )

    @property
    def binary(self) -> str:
        """Path to the binary to execute."""
        return self._binary

    @property
    def opts(self) -> List[Option]:
        """List of options passed to the binary."""
        return deepcopy(self._opts)

    @opts.setter
    def opts(self, value: List[Option]) -> None:
        self._opts = deepcopy(value)

    @property
    def cwd(self) -> str:
        """Path to the directory where the command is executed."""
        return self._cwd

    @property
    def out(self) -> str:
        """Represents the stdout of the last executed command.
        Default is None.
        """
        return self._out

    @property
    def err(self) -> str:
        """Represents the stderr of the last executed command.
        Default is None.
        """
        return self._err

    @property
    def returncode(self) -> int:
        """Represents the return code of the last executed command.
        Default is None.
        """
        return self._returncode



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pytcm']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'pytcm',
    'version': '0.5.1',
    'description': 'pytcm - Python Terminal Commands Manager',
    'long_description': '# pytcm\n\nA Python Terminal Commands Manager\n\n## Installation\n\n```\n$ pip install pytcm\n```\n\n## Usage\n\n### Using execute directly\n\n``` python\nimport pytcm\n\nbinary = \'python\'\nopts = [\n    pytcm.Flag(\'--version\', True)\n]\n\nresult = pytcm.execute(binary, opts)\n\nprint(result.out)  # "Python 3.9.7"\nprint(result.err)  # ""\nprint(result.returncode)  # 0\n```\n\n### Using a Command object that holds the context\n\n``` python\nimport pytcm\n\nbinary = \'python\'\nopts = [\n    pytcm.Flag(\'--version\', True)\n]\n\ncmd = pytcm.Command(binary, opts)\ncmd.execute()\n\nprint(cmd.out)  # "Python 3.9.7"\nprint(cmd.err)  # ""\nprint(cmd.returncode)  # 0\n```\n\n## Options\n\n### Flag\n\nA boolean option\n\n```python\nimport pytcm\n\nflag = pytcm.Flag("--verbose", True)\nopt = flag.parse()\n\nprint(opt)  # "--verbose"\n```\n\n### Explicit\n\nAn option with an equal sign\n\n```python\nimport pytcm\n\nexplicit = pytcm.Explicit("--age", 12)\nopt = explicit.parse()\n\nprint(opt)  # "--age=12"\n```\n\n### Implicit\n\nAn option separated by a space character\n\n```python\nimport pytcm\n\nimplicit = pytcm.Implicit("--age", 12)\nopt = implicit.parse()\n\nprint(opt)  # "--age 12"\n```\n\n### Positional\n\nA simple inline option\n\n```python\nimport pytcm\n\npositional = pytcm.Positional("test.txt")\nopt = positional.parse()\n\nprint(opt)  # "test.txt"\n```\n\n## Contributing\n\nThank you for considering making pytcm better.\n\nPlease refer to [docs](docs/CONTRIBUTING.md).\n\n## Change Log\n\nSee [CHANGELOG](CHANGELOG.md)\n\n## License\n\nMIT',
    'author': 'Alexis Beaulieu',
    'author_email': 'alexisbeaulieu97@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/alexisbeaulieu97/pytcm',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
